-- |Representation of a game stage.

module Game.LupupuCube.Stage
    ( Stage(..)
    , ppStage
    )
    where

import           Game.LupupuCube.Phase
import           Game.LupupuCube.XmlIO
import           Prelude hiding (Left, Right)
import           Text.XML.HXT.Arrow

-- |'Stage' is a representation of a game stage.
data Stage
    = Stage {
        stTitle   :: !String -- ^The human-readable title of the stage.
      , stSize    :: !Size   -- ^The size of the stage.
      , stInitial :: !Phase  -- ^The initial state of the stage.
      }
    deriving (Eq, Show)

instance XmlRead Stage where
    xmlRead = proc e ->
              do title   <- getAttrValue0 "title" -< e
                 size    <- xmlRead -< e
                 initial <- getChildren >>> isElem >>> hasName "initialState" >>> xmlRead -< e
                 returnA -< Stage {
                               stTitle   = title
                             , stSize    = size
                             , stInitial = initial
                             }

instance XmlWrite Stage where
    xmlWrite = proc (Stage title size initial) ->
               do tit <- attr "title" mkText -< title
                  siz <- listA xmlWrite      -< size
                  ini <- eelem "initialState" += xmlWrite -< initial
                  com <- mkCmt -< "The quick sketch of the initial state:\n" ++ ppPhase size initial
                  unlistA -< tit : com : ini : siz

-- |Returns a brief representation of the given 'Stage' in an ASCII
-- art.
ppStage :: Stage -> String
ppStage st = stTitle st ++ "\n" ++ ppPhase (stSize st) (stInitial st)
