module Game.LupupuCube.UI.PhaseWidget
    ( PhaseWidget(..)
    )
    where

import           Control.Monad.Writer
import qualified Data.Map as M
import           Game.LupupuCube.Cube
import           Game.LupupuCube.Phase
import           UI.HSCurses.Curses
import           UI.HSCurses.Widgets hiding (Size)
import           UI.HSCurses.CursesHelper


data PhaseWidget 
    = PhaseWidget {
        pwSize  :: Size
      , pwPhase :: Phase
      }

instance Widget PhaseWidget where
    draw (y, x) _ _ pw = drawPhase (Pos x y) (pwSize pw) (pwPhase pw)
    minSize pw         = (sHeight (pwSize pw), sWidth (pwSize pw))


drawPhase :: Position -> Size -> Phase -> IO ()
drawPhase (Pos x y) sz p = mapM_ drawRow [0 .. sHeight sz - 1]
    where
      drawRow :: Int -> IO ()
      drawRow row = mapM_ (drawCol row) [0 .. sWidth sz - 1]

      drawCol :: Int -> Int -> IO ()
      drawCol row col
          | Pos col row == pSaladPos p
              = do setStyle saladAttr
                   mvWAddStr stdScr (rowToY row) (colToX col) "@"
          | otherwise
              = case M.lookup (Pos col row) (pMatrix p) of
                  Nothing
                      -> do setStyle bgAttr
                            mvWAddStr stdScr (rowToY row) (colToX col) "."
                  Just cube
                      -> drawCube (rowToY row) (colToX col) cube

      rowToY :: Int -> Int
      rowToY = (-) (y + sHeight sz - 1)

      colToX :: Int -> Int
      colToX = (+ x)

      saladAttr :: CursesStyle
      saladAttr = defaultCursesStyle
    
      bgAttr :: CursesStyle
      bgAttr = defaultCursesStyle


drawCube :: Int -> Int -> Cube -> IO ()
drawCube y x cube
    = do setStyle (mkCubeStyle cube)
         mvWAddStr stdScr y x (return (ppCube cube))


mkCubeStyle :: Cube -> CursesStyle
mkCubeStyle cube = mkCursesStyle (execWriter w)
    where
      w :: Writer [Attribute] ()
      w = do when (cuIsDowny cube)
                      $ tell [Underline]
             when (cuIsFixed cube)
                      $ tell [Bold]
             when (cuIsLarge cube || isLinkedSomehow cube)
                      $ tell [Reverse]
