-- |A simple layout manager that places just one widget at the center
-- of container widget. The inner widget will always be drawn in its
-- minimum size.

module Game.LupupuCube.UI.SpacingLayout
    ( SpacingLayoutWidget
    , mkSpacingLayout
    )
    where

import           UI.HSCurses.Widgets

-- |This is a widget containing another widget.
data SpacingLayoutWidget
    = forall w. Widget w => SpacingLayoutWidget w

instance Widget SpacingLayoutWidget where
    draw pos sz hint (SpacingLayoutWidget w)
        = let wWidth  = getWidth  (minSize w)
              wHeight = getHeight (minSize w)
              spcX    = fromIntegral (getWidth  sz - wWidth ) / 2 :: Double
              spcY    = fromIntegral (getHeight sz - wHeight) / 2 :: Double
              wX      = floor $ fromIntegral (getXPos pos) + spcX
              wY      = floor $ fromIntegral (getYPos pos) + spcY
          in
            draw (wY, wX) (wHeight, wWidth) hint w

    minSize (SpacingLayoutWidget w)
        = minSize w

-- |Creates 'SpacingLayoutWidget' containing a given widget.
mkSpacingLayout :: forall w. Widget w => w -> SpacingLayoutWidget
mkSpacingLayout = SpacingLayoutWidget